SystemOrganization addCategory: #ScamperM!
SystemOrganization addCategory: #'ScamperM-Parser Entities'!

!ShiftJISTextConverter class methodsFor: '*scamperm' stamp: 'ich. 5/1/2004 16:52'!
checkCode: upperCharacter followed: lowerCharacter
	| u l |
	u _ upperCharacter asInteger.
	u < 16r80 ifTrue: [^ true].
	l _ lowerCharacter asInteger.
	((u between: 16r81 and: 16r9F) | (u between: 16rE0 and: 16rFF))
		& ((l between: 16r40 and: 16rFC) & (l ~= 16r7F)) ifTrue: [^ true].
	^ false.
! !

!ShiftJISTextConverter class methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 22:54'!
encodingNames
	"I found the site that returns x-sjis as charset, so I added it. If acceptable, this method should return to the method category printing. If acceptable, this method should return to the method category utilities.
	ich. 01/05/06"
	^#('shift-jis' 'shift_jis' 'sjis' 'x-sjis' ) copy! !

!ShiftJISTextConverter class methodsFor: '*scamperm' stamp: 'ich. 5/1/2004 16:30'!
isFirstByte: aCharacter
	^ (aCharacter asInteger between: 16r81 and: 16r9F) | (aCharacter asInteger between: 16rE0 and: 16rEF).
! !

!HtmlHead methodsFor: '*scamperm' stamp: 'ich. 5/1/2004 16:49'!
charset
	self allSubentitiesDo:
		[:each |
		(each isMemberOf: HtmlMeta) ifTrue:
			[| retval |
			retval _ each charset.
			retval ifNotNil: [^ retval]]].
	^ nil.
! !

Object subclass: #Bookmark
	instanceVariableNames: 'dictionary directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM'!

!Bookmark commentStamp: '<historical>' prior: 0!
It is a class representing a web browser's bookmark or favorites.

The actual data is stored in a directory (by default, './Bookmark') as *.lin files.!

!Bookmark methodsFor: 'private' stamp: 'ich. 12/29/2005 21:20'!
addMenuTo: menu for: aWebRetriever 
	| sub |
	dictionary keysAndValuesDo: 
			[:name :value | 
			(value isKindOf: Dictionary) 
				ifTrue: 
					[sub := self addNewSubMenu: value for: aWebRetriever.
					menu add: name subMenu: sub]
				ifFalse: 
					[menu 
						add: name
						selector: #jumpToUrl:
						argument: value]]! !

!Bookmark methodsFor: 'private' stamp: 'ich. 1/3/2006 13:40'!
addNewSubMenu: aDictionary for: aWebRetriever 
	| subMenu url sub |
	subMenu := MenuMorph new.
	aDictionary keysAndValuesDo: 
			[:name :value | 
			url := value.
			(url isKindOf: Dictionary) 
				ifTrue: 
					[sub := self addNewSubMenu: url for: aWebRetriever.
					subMenu add: name subMenu: sub]
				ifFalse: 
					[subMenu 
						add: name
						target: aWebRetriever
						selector: #jumpToUrl:
						argument: url]].
	^subMenu! !

!Bookmark methodsFor: 'adding' stamp: 'ich. 12/28/2005 01:06'!
add: anAssociation
	^ dictionary add: anAssociation.
! !

!Bookmark methodsFor: 'private' stamp: 'ich. 12/29/2005 21:12'!
bookFrom: aDirectory 
	| book |
	book := Dictionary new.
	aDirectory directoryNames do: 
		[:dn | 
		book add: (Association
			key: dn
			value: (self bookFrom: (aDirectory directoryNamed: dn)))].
	aDirectory fileNames do: 
		[:fn | 
		('*.lin' match: fn) ifTrue: 
			[book add: (Association
				key: (fn truncateTo: fn size - '.lin' size)
				value: (self readUrlFromFile: (aDirectory fullNameFor: fn)))].
		('*.url' match: fn)  ifTrue: 
			[book add: (Association
				key: (fn truncateTo: fn size - '.url' size)
				value: (self importDotUrlFile: (aDirectory fullNameFor: fn)))]].
	^book! !

!Bookmark methodsFor: 'private' stamp: 'ich. 12/22/2005 01:42'!
createBookmarkFiles: aDirectory dict: aDictionary dirname: aName
	| dir |
	(aDirectory directoryExists: aName) ifFalse:[aDirectory createDirectory: aName]. 
	dir := aDirectory directoryNamed: aName.
	aDictionary keysAndValuesDo:
		[:k :v |
		(v isKindOf: Dictionary)
			ifTrue:[self createBookmarkFiles: dir dict: v dirname: k]
			ifFalse:[self save: k, '.lin' forLinkOf: v.]].
! !

!Bookmark methodsFor: 'private' stamp: 'ich. 12/22/2005 01:38'!
defaultDirectoryName
	^ 'Bookmark'.
! !

!Bookmark methodsFor: 'adding' stamp: 'ich. 12/29/2005 23:13'!
import
	| newDirectory |
	newDirectory := FillInTheBlank request: 'Directory to import' translated
				initialAnswer: directory pathName.
	(newDirectory isNil or: [newDirectory isEmpty]) ifTrue: [^self].
	self importFrom: newDirectory! !

!Bookmark methodsFor: 'private' stamp: 'ich. 12/29/2005 23:17'!
importDotUrlFile: aFineName 
	| oldFile urlLine |
	oldFile := FileStream oldFileOrNoneNamed: aFineName.
	oldFile isBinary 	ifTrue: [self error: 'not url file' translated. ^ nil].
	urlLine := oldFile contentsOfEntireFile substrings
		detect: [:sub |  (sub beginsWith: 'URL=')]
		ifNone: [^ nil].
	^ (urlLine copyFrom: 'URL=' size + 1 to: urlLine size) asUrl.
! !

!Bookmark methodsFor: 'private' stamp: 'ich. 12/30/2005 00:44'!
importFrom: newDirectory 
	(FileDirectory default directoryExists: newDirectory) ifFalse: [^ self].

	(self bookFrom: (FileDirectory on: newDirectory)) associationsDo: 
		[:ass | 
		self add: ass.
		(ass value isKindOf: Dictionary) 
			ifTrue: 
				[self 
					createBookmarkFiles: directory
					dict: ass value
					dirname: ass key]
			ifFalse: [self save: ass key , '.lin' forLinkOf: ass value]].
! !

!Bookmark methodsFor: 'initialize-release' stamp: 'ich. 12/24/2005 18:35'!
initialize
	dictionary := Dictionary new.
	self load.

! !

!Bookmark methodsFor: 'adding' stamp: 'ich. 12/28/2005 01:01'!
load
	dictionary := (FileDirectory default directoryExists: self defaultDirectoryName) 
		ifTrue: 
			[directory := FileDirectory default directoryNamed: self defaultDirectoryName.
			self bookFrom: directory]
		ifFalse: 
			[FileDirectory default createDirectory: self defaultDirectoryName.
			directory := FileDirectory default directoryNamed: self defaultDirectoryName.
			Dictionary new]! !

!Bookmark methodsFor: 'private' stamp: 'ich. 12/24/2005 19:59'!
readUrlFromFile: aFile
	| oldFile url |
	oldFile := FileStream oldFileOrNoneNamed: aFile.
	oldFile isBinary 
		ifTrue:[ self error: 'not url file' translated]
		ifFalse:[url := (oldFile contentsOfEntireFile) withBlanksTrimmed.
				url =='' ifTrue:[ self error: 'blank file: url not exist' translated].
				^url asUrl].
	! !

!Bookmark methodsFor: 'private' stamp: 'ich. 12/22/2005 01:11'!
save: aFilename forLinkOf: aUrl
	| file |
	directory deleteFileNamed: aFilename. 
	file := StandardFileStream fileNamed: (directory fullNameFor: aFilename).
	file ifNil:[self error: 'could not save file' translated].
	file nextPutAll: aUrl asString. 
	file close.
! !

!Bookmark methodsFor: 'menus' stamp: 'ich. 12/29/2005 21:20'!
showMenuFor: aWebRetriever 
	| menu |
	menu := (MenuMorph entitled: ' Bookmark ' translated) 
				defaultTarget: aWebRetriever.
	menu addStayUpItem.
	menu addLine.
	menu 
		add: 'add to bookmark' translated
		target: aWebRetriever
		selector: #addToBookmark.
	menu 
		add: 'Import...' translated
		target: aWebRetriever
		selector: #importBookmark.
	menu addLine.
	self addMenuTo: menu for: aWebRetriever.
	menu popUpInWorld: aWebRetriever currentWorld! !

Object subclass: #ScamperWorker
	instanceVariableNames: 'items boss'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM'!

!ScamperWorker commentStamp: '<historical>' prior: 0!
I work in the background to help download the morphs that are not completed in a page such as images.
!

!ScamperWorker class methodsFor: 'intance creation' stamp: 'ckl 8/14/2005 11:42'!
task: morphs from: aScamper
	^self new setTask: morphs from: aScamper
! !

!ScamperWorker methodsFor: 'private' stamp: 'ckl 8/14/2005 11:42'!
setTask: morphs from: aScamper
	items _ morphs.
	boss _ aScamper
! !

!ScamperWorker methodsFor: 'work' stamp: 'ckl 8/14/2005 11:51'!
start
	[items do: [:m | m downloadStateIn: boss]] fork
! !

Object subclass: #WebHistory
	instanceVariableNames: 'recentDocuments currentUrlIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM'!

!WebHistory commentStamp: '<historical>' prior: 0!
It is a class representing a history of a web browser.!

!WebHistory methodsFor: 'accessing' stamp: 'ich. 1/4/2006 23:06'!
add: mimeDocument 
	currentUrlIndex > recentDocuments size 
		ifTrue: [recentDocuments addLast: mimeDocument].
	recentDocuments size > self maxHistoryLength ifTrue:
		[recentDocuments removeFirst.
		currentUrlIndex := currentUrlIndex - 1]
! !

!WebHistory methodsFor: 'accessing' stamp: 'ich. 1/3/2006 13:58'!
back
	currentUrlIndex <= 1 ifTrue: [^nil].
	currentUrlIndex := currentUrlIndex - 1.
	currentUrlIndex <= recentDocuments size ifTrue:
		[^ recentDocuments at: currentUrlIndex].
	^ nil.
! !

!WebHistory methodsFor: 'accessing' stamp: 'ich. 1/4/2006 22:33'!
clearForNewPage
	"delete all elements in recentDocuments after currentUrlIndex"
	recentDocuments removeLast: (recentDocuments size - currentUrlIndex max: 0).
	currentUrlIndex := currentUrlIndex + 1.
! !

!WebHistory methodsFor: 'accessing' stamp: 'ich. 1/3/2006 13:58'!
forward
	currentUrlIndex >= recentDocuments size ifTrue: [^ nil].
	currentUrlIndex := currentUrlIndex + 1.
	^ recentDocuments at: currentUrlIndex.
! !

!WebHistory methodsFor: 'initialize-release' stamp: 'ich. 12/22/2005 01:47'!
initialize
	recentDocuments _ OrderedCollection new.
	currentUrlIndex _ 0.
! !

!WebHistory methodsFor: 'private' stamp: 'ich. 1/3/2006 13:55'!
maxHistoryLength
	^20! !

!WebHistory methodsFor: 'user interface' stamp: 'ich. 1/3/2006 13:54'!
menuFor: aWebRetriever selector: aSymbol 
	"Let the user selecet a previous page to view."

	| menu |
	menu := MenuMorph entitled: 'Recent URLs' translated.
	menu defaultTarget: aWebRetriever.
	menu addStayUpItem.
	menu addLine.
	recentDocuments reverseDo: 
			[:doc | 
			menu 
				add: doc url toText
				selector: aSymbol
				argument: doc].
	menu popUpInWorld: self currentWorld! !

!HtmlTag class methodsFor: '*scamperm' stamp: 'ich 1/17/2006 22:41'!
entityClasses
	"I added HtmlScript for script tag. If acceptable, this method should return to the method category parser support with movement of HtmlScript From ScamperM to Network-HTML-Parser Entities.
	ich. 01/05/06"
	"a Dictionary mapping tag names into the correct entity class"

	"EntityClasses _ nil"

	EntityClasses isNil ifFalse: [^EntityClasses].
	EntityClasses := Dictionary new.
	#(
		#body
		#HtmlBody
		#head
		#HtmlHead
		#html
		#HtmlHtml
		#noscript
		#HtmlNoScript
		#script
		#HtmlScript
		#frameset
		#HtmlFrame
		#frame
		#HtmlFrame
		#title
		#HtmlTitle
		#style
		#HtmlStyle
		#meta
		#HtmlMeta
		#p
		#HtmlParagraph
		#form
		#HtmlForm
		#blockquote
		#HtmlBlockQuote
		#input
		#HtmlInput
		#textarea
		#HtmlTextArea
		#select
		#HtmlSelect
		#optgroup
		#HtmlOptionGroup
		#option
		#HtmlOption
		#img
		#HtmlImage
		#embed
		#HtmlEmbedded
		#noembed
		#HtmlNoEmbed
		#a
		#HtmlAnchor
		#br
		#HtmlBreak
		#map
		#HtmlMap
		#area
		#HtmlArea
		#li
		#HtmlListItem
		#dd
		#HtmlDefinitionDefinition
		#dt
		#HtmlDefinitionTerm
		#ol
		#HtmlOrderedList
		#ul
		#HtmlUnorderedList
		#dl
		#HtmlDefinitionList
		#h1
		#HtmlHeader
		#h2
		#HtmlHeader
		#h3
		#HtmlHeader
		#h4
		#HtmlHeader
		#h5
		#HtmlHeader
		#h6
		#HtmlHeader
		#hr
		#HtmlHorizontalRule
		#strong
		#HtmlBoldEntity
		#b
		#HtmlBoldEntity
		#em
		#HtmlItalicsEntity
		#i
		#HtmlItalicsEntity
		#dfn
		#HtmlItalicsEntity
		#u
		#HtmlUnderlineEntity
		#tt
		#HtmlFixedWidthEntity
		#kbd
		#HtmlFixedWidthEntity
		#strike
		#HtmlStrikeEntity
		#big
		#HtmlBiggerFontEntity
		#small
		#HtmlSmallerFontEntity
		#sub
		#HtmlSubscript
		#sup
		#HtmlSuperscript
		#font
		#HtmlFontEntity
		#pre
		#HtmlPreformattedRegion
		#table
		#HtmlTable
		#tr
		#HtmlTableRow
		#td
		#HtmlTableDataItem
		#th
		#HtmlTableHeader
	) 
			pairsDo: [:tagName :className | EntityClasses at: tagName asString put: (Smalltalk at: className)].
	^EntityClasses! !

!HtmlTag methodsFor: '*scamperm' stamp: 'ich 1/16/2006 20:21'!
entityFor
	"return an empty entity corresponding to this tag"
	| eClass |
	eClass _ self class entityClasses
		at: name
		ifAbsent: [^HtmlNotSupportedYet new tagName: name].
	^eClass forTag: self ! !

!WebPageMorph methodsFor: '*scamperm' stamp: 'ich. 1/3/2006 11:45'!
stepAt: millisecondClockValue
	model ifNotNil: [model stepAt: millisecondClockValue in: self].
	super stepAt: millisecondClockValue "let player, if any, step"
! !

!WebPageMorph methodsFor: '*scamperm' stamp: 'ich. 1/3/2006 11:45'!
wantsSteps
	"Return true if the model wants its view to be stepped.  For an open system window, we give the model to offer an opinion"

	self isPartsDonor ifTrue: [^ false].
	self player wantsSteps ifTrue: [^ true].
	^ model wantsSteps.! !

TestCase subclass: #ScamperMRelatedMethodsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM'!

TestCase subclass: #ScamperMTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM'!

!ScamperMTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Scamper. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!ScamperMTest methodsFor: 'initialize-release' stamp: 'ich. 1/5/2006 20:50'!
testOpen
	| scamper |
	self shouldnt: [scamper := ScamperM open] raise: Error.
	scamper delete! !

!HTTPSocket class methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 22:23'!
expandUrl: newUrl ip: byteArrayIP port: portNum 
	"I found the site whose newUrl begins with /, so I treated it same as newUrl begins with ../. If acceptable, this method should return to the method category utilities.
	ich. 01/05/06"
	(newUrl beginsWith: '/') | (newUrl beginsWith: '../') ifFalse: [^newUrl].
	^String streamContents: 
			[:strm | 
			byteArrayIP do: 
					[:bb | 
					bb printOn: strm.
					strm nextPut: $.].
			strm
				skip: -1;
				nextPut: $:.
			portNum printOn: strm.
			strm nextPutAll: (newUrl 
						allButFirst: ((newUrl beginsWith: '../') ifTrue: [2] ifFalse: [0]))]! !

!FreeTranslation class methodsFor: '*scamperm' stamp: 'tak 7/7/2004 09:44'!
openGooDictionaryScamperOn: currentSelection 
	"Submit the string to the translation server at dictionary.goo.ne.jp."
	| inputs scamperWindow |
	inputs _ Dictionary new.
	inputs at: 'MT' put: {currentSelection}.
	inputs at: 'kind' put: #('ej' ).
	scamperWindow _ (WebBrowser default
				ifNil: [^ self])
				newOrExistingOn: 'http://dictionary.goo.ne.jp/search.php'.
	scamperWindow model
		submitFormWithInputs: inputs
		url: 'http://dictionary.goo.ne.jp/search.php' asUrl
		method: 'GET'.
	scamperWindow activate! !

!FreeTranslation class methodsFor: '*scamperm' stamp: 'tak 7/7/2004 09:41'!
openScamperOn: currentSelection
	"Submit the string to the translation server at www.freetranslation.com.  Ask it to translate from (Preferences parameterAt: #languageTranslateFrom) to (Preferences parameterAt: #languageTranslateTo).  Display the results in a Scamper window, reusing the previous one if possible."

	| inputs scamperWindow from to | 
	currentSelection size >= 10000 ifTrue: [^ self inform: 'Text selection is too long.'].
	from _ Preferences parameterAt: #languageTranslateFrom ifAbsentPut: ['English'].
	to _ Preferences parameterAt: #languageTranslateTo ifAbsentPut: ['German'].
	from = to ifTrue:
			[^ self inform: 'You asked to translate from ', from, ' to ', to, '.\' withCRs,
				'Use "choose language" to set these.'].
	to = 'Japanese' ifTrue: [ ^self openGooDictionaryScamperOn: currentSelection ].
	inputs _ Dictionary new.
	inputs at: 'SrcText' put: (Array with: currentSelection).
	inputs at: 'Sequence' put: #('core').
	inputs at: 'Mode' put: #('html').
	inputs at: 'template' put: #('TextResult2.htm').
	inputs at: 'Language' put: (Array with: from, '/', to).
	scamperWindow _ (WebBrowser default ifNil: [^self]) newOrExistingOn: 'http://ets.freetranslation.com'.
	scamperWindow model submitFormWithInputs: inputs 
		url: 'http://ets.freetranslation.com:5081' asUrl
		method: 'post'.
	scamperWindow activate.
! !

!HtmlCommentEntity methodsFor: '*scamperm' stamp: 'ich 1/16/2006 01:29'!
doScriptOn: aStream indent: indent
	self printHtmlOn: aStream indent: indent.
! !

!HtmlCommentEntity methodsFor: '*scamperm' stamp: 'ich 1/17/2006 22:57'!
printHtmlOn: aStream indent: indent 
	indent timesRepeat: [aStream space].
	aStream nextPutAll: self commentText. "comentText already includes comment elements."
	aStream cr! !

!WideString methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 23:01'!
encodeForHTTP
	"change dangerous characters to their %XX form, for use in HTTP transactions"

	^ self encodeForHTTPWithTextEncoding: 'utf-8' conditionBlock: [:c | c isSafeForHTTP].
! !

HtmlEntity subclass: #HtmlEntityWithNoEndTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

HtmlEntityWithNoEndTag subclass: #HtmlArea
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlArea methodsFor: 'formatting' stamp: 'bolot 2/27/2000 22:18'!
buildMorph
	"construct a hot-spot morph"
	| coords vertices radiusX radiusY |
	coords _ (self coords findTokens: ', ') collect: [:elem | elem asNumber asInteger].
	self shape isEmptyOrNil
		ifTrue: [^nil].

	(self shape asLowercase beginsWith: 'poly')
		ifTrue: [coords size even ifFalse: [^nil].
			vertices _ OrderedCollection new.
			coords pairsDo: [:x :y |
				vertices add: x @ y].
			^(PolygonMorph vertices: vertices color: Color transparent
				borderWidth: 1 borderColor: Color transparent) quickFill: false; makeClosed].

	(coords size > 4 or: [coords size < 3])
		ifTrue: [^nil].

	self shape asLowercase = 'circle'
		ifTrue: [radiusX _ coords third.
			radiusY _ coords last.
			^(EllipseMorph newBounds:
				(((coords first - radiusX) @ (coords second - radiusY))
				extent:
				((2 * radiusX) @ (2 * radiusY)))
			color: Color transparent) borderColor: Color transparent].

	coords size = 4
		ifFalse: [^nil].

	(self shape asLowercase beginsWith: 'rect')
		ifTrue: [^(RectangleMorph newBounds:
				(Rectangle origin: (coords first @ coords second)
				corner: (coords third @ coords last))
			color: Color transparent) borderColor: Color transparent].

	^nil! !

!HtmlArea methodsFor: 'accessing' stamp: 'bolot 11/3/1999 17:48'!
coords
	^self getAttribute: 'coords'! !

!HtmlArea methodsFor: 'accessing' stamp: 'bolot 11/3/1999 17:48'!
href
	^self getAttribute: 'href'! !

!HtmlArea methodsFor: 'testing' stamp: 'bolot 11/4/1999 04:43'!
isArea
	^true! !

!HtmlArea methodsFor: 'formatting' stamp: 'ar 3/17/2001 14:25'!
linkMorphForMap: map andBrowser: browser
	| m |
	(m _ self buildMorph) ifNil: [^nil].
	m color: (Color random alpha: 0.1). "hack to ensure the morph is clickable"
	m
		on: #mouseUp
		send: #mouseUpBrowserAndUrl:event:linkMorph:
		to: map
		withValue: {browser. self href}.
	^m! !

!HtmlArea methodsFor: 'testing' stamp: 'bolot 11/3/1999 18:06'!
mayContain: anEntity
	^false! !

!HtmlArea methodsFor: 'accessing' stamp: 'bolot 11/3/1999 17:48'!
shape
	^self getAttribute: 'shape'! !

!HtmlArea methodsFor: 'testing' stamp: 'bolot 11/3/1999 17:47'!
tagName
	^'area'! !

HtmlEntityWithNoEndTag subclass: #HtmlBaseFontSize
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

HtmlEntityWithNoEndTag subclass: #HtmlBaseURI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

HtmlEntityWithNoEndTag subclass: #HtmlBreak
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlBreak methodsFor: 'formatting' stamp: 'ls 7/17/1998 19:21'!
addToFormatter: formatter
	formatter addChar: Character cr.! !

!HtmlBreak methodsFor: 'testing' stamp: 'ls 7/4/1998 16:14'!
mayContain: anEntity
	^false! !

!HtmlBreak methodsFor: 'testing' stamp: 'ls 7/4/1998 16:14'!
tagName
	^'br'! !

!HtmlEntityWithNoEndTag methodsFor: 'as yet unclassified' stamp: 'ich 1/16/2006 01:22'!
doScriptOn: aStream indent: indent
	aStream next: indent put: $ ;
	nextPutAll: '<';
	 nextPutAll: self tagName.
	self attributes associationsDo: [:assoc | aStream space; nextPutAll: assoc key; nextPutAll: '="'; nextPutAll: assoc value; nextPutAll: '"'].
	aStream nextPut: $>;
	 cr.
	contents do: [:entity | entity doScriptOn: aStream indent: indent + 1].
! !

HtmlEntityWithNoEndTag subclass: #HtmlFrame
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlFrame methodsFor: 'formatting' stamp: 'ls 7/26/1998 21:53'!
addToFormatter: formatter
	| src |
	src _ self getAttribute: 'src' default: nil.
	formatter ensureNewlines: 1.
	src ifNotNil: [ formatter startLink: src ].
	formatter addString: 'frame '.
	formatter addString: (self name ifNil: ['(unnamed)']).
	src ifNotNil:  [ formatter endLink: src ].
	formatter ensureNewlines: 1.! !

!HtmlFrame methodsFor: 'categorization' stamp: 'ls 7/29/1998 00:25'!
isFrame
	^true! !

!HtmlFrame methodsFor: 'testing' stamp: 'ls 7/26/1998 21:46'!
mayContain: anEntity
	^false! !

!HtmlFrame methodsFor: 'testing' stamp: 'ls 7/26/1998 21:46'!
tagName
	^'frame'! !

HtmlEntityWithNoEndTag subclass: #HtmlHorizontalRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlHorizontalRule methodsFor: 'formatting' stamp: 'ls 6/27/1998 15:57'!
addToFormatter: formatter
	formatter hr! !

!HtmlHorizontalRule methodsFor: 'testing' stamp: 'ls 6/27/1998 15:52'!
mayContain: anEntity
	^false! !

!HtmlHorizontalRule methodsFor: 'testing' stamp: 'ls 6/25/1998 02:22'!
tagName
	^'hr'! !

HtmlEntityWithNoEndTag subclass: #HtmlImage
	instanceVariableNames: 'image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlImage methodsFor: 'formatting' stamp: 'bolot 2/27/2000 23:45'!
addToFormatter: formatter
	| morph url |
	self src isNil ifTrue: [ ^self ].
	url _ self src.
	formatter baseUrl ifNotNil: [ 
		url _ url asUrlRelativeTo: formatter baseUrl ].


	morph _ DownloadingImageMorph new.
	morph defaultExtent: self imageExtent.
	morph altText: self alt.
	morph url: url.
	self imageMapName
		ifNotNil:
			[morph imageMapName: self imageMapName.
			morph formatter: formatter].

	formatter addIncompleteMorph: morph.! !

!HtmlImage methodsFor: 'attributes' stamp: 'ls 8/4/1998 21:58'!
alt
	^(self getAttribute: 'alt') ifNil: ['[image]']! !

!HtmlImage methodsFor: 'downloading' stamp: 'ls 8/8/1998 03:47'!
downloadState: baseUrl 
	|  sourceUrl imageSource |

	image ifNil: [ 
		sourceUrl _ self src.
		sourceUrl ifNotNil: [ 
			imageSource _ HTTPSocket httpGetDocument: (sourceUrl asUrlRelativeTo: baseUrl asUrl) toText.
			imageSource contentType = 'image/gif'  ifTrue: [
				[image _ (GIFReadWriter on: (RWBinaryOrTextStream with: imageSource content) reset binary) nextImage ]
				ifError: [ :a :b |  "could not decode--ignore it"  image _ nil ] ].
			 ] ].
! !

!HtmlImage methodsFor: 'attributes' stamp: 'ls 10/22/2001 11:14'!
imageExtent
	"the image extent, according to the WIDTH and HEIGHT attributes.  returns nil if either WIDTH or HEIGHT is not specified"
	| widthText heightText |
	widthText _ self getAttribute: 'width' ifAbsent: [ ^nil ].
	heightText _ self getAttribute: 'height' ifAbsent: [ ^nil ].
	^ [ widthText asNumber @ heightText asNumber ] ifError: [ :a :b | nil ]! !

!HtmlImage methodsFor: 'attributes' stamp: 'bolot 11/30/1999 00:42'!
imageMapName
	| imageMapName |
	(imageMapName _ self getAttribute: 'usemap')
		ifNil: [^nil].
	imageMapName first = $#
		ifTrue: [imageMapName _ imageMapName copyFrom: 2 to: imageMapName size].
	^imageMapName! !

!HtmlImage methodsFor: 'initializing' stamp: 'ls 7/27/1998 02:17'!
initialize: aTag	
	super initialize: aTag.
! !

!HtmlImage methodsFor: 'testing' stamp: 'ls 7/1/1998 02:30'!
mayContain: anEntity
	^false! !

!HtmlImage methodsFor: 'attributes' stamp: 'ls 7/29/1998 00:56'!
src
	^self getAttribute: 'src' default: nil! !

!HtmlImage methodsFor: 'testing' stamp: 'ls 6/25/1998 02:22'!
tagName
	^'img'! !

HtmlEntityWithNoEndTag subclass: #HtmlInput
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlInput commentStamp: '<historical>' prior: 0!
result of an <input> tag!

!HtmlInput methodsFor: 'formatting' stamp: 'bf 11/4/1999 21:29'!
addCheckBoxButtonToFormatter: formatter
	| name formData checked button buttonInput |

	"dig up relevant attributes"
	name _ self getAttribute: 'name'.
	name ifNil: [ ^self ].
	value _ self getAttribute: 'value'.
	value ifNil: [ ^value ].
	
	formData _ formatter currentFormData.
	formData ifNil:  [ ^self ].

	checked _ (self getAttribute: 'checked') isNil not.

	"set up the form input"
	buttonInput _ ToggleButtonInput name: name value: value checkedByDefault: checked.
	formData addInput: buttonInput.

	"create the actual button"
	button _ UpdatingThreePhaseButtonMorph checkBox.
	button target: buttonInput;
		getSelector: #pressed;
		actionSelector: #toggle.
	buttonInput button: button.
	formatter addMorph: button.


! !

!HtmlInput methodsFor: 'formatting' stamp: 'bolot 11/27/1999 18:47'!
addFileInputToFormatter: formatter
	"is it a submit button?"
	| inputMorph size fileInput |
	inputMorph _ PluggableTextMorph on: StringHolder new text: #contents accept: #acceptContents:.
	size _ (self getAttribute: 'size' default: '12') asNumber.
	inputMorph extent: (size*10@20).
	fileInput _ FileInput name: self name textMorph: inputMorph.
	formatter addMorph: inputMorph;
		addMorph: ((PluggableButtonMorph on: fileInput getState: nil action: #browse)
				label: 'Browse').
	formatter currentFormData addInput: fileInput! !

!HtmlInput methodsFor: 'formatting' stamp: 'ar 3/17/2001 14:26'!
addImageButtonToFormatter: formatter
	"is it a submit button?"
	| formData imageUrl morph |
	(imageUrl _ self getAttribute: 'src') ifNil: [^self].
	formatter baseUrl
		ifNotNil: [imageUrl _ imageUrl asUrlRelativeTo: formatter baseUrl].

	morph _ DownloadingImageMorph new.
	morph defaultExtent: self imageExtent.
	morph altText: self alt.
	morph url: imageUrl.

	value _ self getAttribute: 'name' default: 'Submit'.
	formData _ formatter currentFormData.
	morph
		on: #mouseUp
		send: #mouseUpFormData:event:linkMorph:
		to: self
		withValue: formData.
	formatter addIncompleteMorph: morph
! !

!HtmlInput methodsFor: 'formatting' stamp: 'bf 11/4/1999 21:47'!
addRadioButtonToFormatter: formatter
	| name formData checked buttonSet button buttonInput |

	"dig up relevant attributes"
	name _ self getAttribute: 'name'.
	name ifNil: [ ^self ].
	value _ self getAttribute: 'value'.
	value ifNil: [ ^value ].
	
	formData _ formatter currentFormData.
	formData ifNil:  [ ^self ].

	checked _ self getAttribute: 'checked'.


	"find or create the set of buttons with our same name"
	buttonSet _ formData inputs detect: [ :i | i isRadioButtonSetInput and: [ i name = name ] ] ifNone: [ nil ].
	buttonSet ifNil: [ 
		"create a new button set"
		buttonSet _ RadioButtonSetInput name: name.
		formData addInput: buttonSet. ].

	"set up the form input"
	buttonInput _ RadioButtonInput  inputSet: buttonSet value: value.
	buttonSet addInput: buttonInput.
	checked ifNotNil: [
		buttonSet  defaultButton: buttonInput ].

	"create the actual button"
	button _ UpdatingThreePhaseButtonMorph radioButton.
	button target: buttonInput;
		getSelector: #pressed;
		actionSelector: #toggle.
	buttonInput button: button.
	formatter addMorph: button.


! !

!HtmlInput methodsFor: 'formatting' stamp: 'bolot 11/2/1999 14:20'!
addTextInputToFormatter: formatter
	"is it a submit button?"
	| inputMorph size |
	inputMorph _ PluggableTextMorph on: StringHolder new text: #contents accept: #acceptContents:.
	self type = 'password'
		ifTrue: [inputMorph font: (StrikeFont passwordFontSize: 12)].
	size _ (self getAttribute: 'size' default: '12') asNumber.
	inputMorph extent: (size*10@20).
	formatter addMorph: inputMorph.
	formatter currentFormData addInput:
		(TextInput name: self name defaultValue: self defaultValue  textMorph: inputMorph).! !

!HtmlInput methodsFor: 'formatting' stamp: 'ls 10/28/2003 11:46'!
addToFormatter: formatter
	formatter currentFormData ifNil: [
		"not in a form.  It's bogus HTML but try to survive"
		^self ].

	"is it a submit button?"
	self type = 'submit' ifTrue: [
		formatter addMorph: ((PluggableButtonMorph on: formatter currentFormData getState: nil action: #submit) label: (self getAttribute: 'value' default: 'Submit')).
		^self ].

	self type = 'image'
		ifTrue: [^self addImageButtonToFormatter: formatter].

	(self type = 'text' or: [self type = 'password'])
		ifTrue: [^self addTextInputToFormatter: formatter].

	self type = 'hidden' ifTrue: [
		formatter currentFormData addInput: (HiddenInput name: self name  value: self defaultValue).
		^self ].

	self type = 'radio' ifTrue: [ 
		^self addRadioButtonToFormatter: formatter ].

	self type = 'checkbox' ifTrue: [ 
		^self addCheckBoxButtonToFormatter: formatter ].

	self type = 'file' ifTrue: [ 
		^self addFileInputToFormatter: formatter ].

	formatter addString: '[form input of type: ', self type, ']'.! !

!HtmlInput methodsFor: 'attributes' stamp: 'ls 9/18/1998 08:22'!
defaultValue
	^(self getAttribute: 'value' default: '') replaceHtmlCharRefs! !

!HtmlInput methodsFor: 'attributes' stamp: 'bolot 11/2/1999 14:14'!
imageExtent
	"the image extent, according to the WIDTH and HEIGHT attributes.  returns nil if either WIDTH or HEIGHT is not specified"
	| widthText heightText |
	widthText _ self getAttribute: 'width' ifAbsent: [ ^nil ].
	heightText _ self getAttribute: 'height' ifAbsent: [ ^nil ].
	^ widthText asNumber @ heightText asNumber! !

!HtmlInput methodsFor: 'testing' stamp: 'ls 7/5/1998 02:15'!
isTextualEntity
	^true! !

!HtmlInput methodsFor: 'testing' stamp: 'ls 7/5/1998 02:14'!
mayContain: anEntity
	^false! !

!HtmlInput methodsFor: 'morphic' stamp: 'ar 3/18/2001 17:27'!
mouseUpEvent: arg1 linkMorph: arg2 formData: arg3
	"Reorder the arguments for existing event handlers"
	(arg3 isMorph and:[arg3 eventHandler notNil]) ifTrue:[arg3 eventHandler fixReversedValueMessages].
	^self mouseUpFormData: arg1 event: arg2 linkMorph: arg3! !

!HtmlInput methodsFor: 'morphic' stamp: 'ar 3/17/2001 14:26'!
mouseUpFormData: formData event: event linkMorph: linkMorph
	| aPoint |
	aPoint _ event cursorPoint - linkMorph topLeft.
	formData addInput: (HiddenInput name: (value, '.x') value: aPoint x asInteger asString).
	formData addInput: (HiddenInput name: (value, '.y') value: aPoint y asInteger asString).
	formData submit! !

!HtmlInput methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 22:38'!
replaceToWideString: aTextConverter 
	| v |
	v := self getAttribute: 'value' ifAbsent: [^self].
	self at: 'value'
		put: (v convertFromWithConverter: aTextConverter) 
! !

!HtmlInput methodsFor: 'testing' stamp: 'ls 7/21/1998 06:51'!
suppliesInput
	"whether we actually have input to supply"
	self type = 'text' ifTrue: [ ^true ].
	^false! !

!HtmlInput methodsFor: 'testing' stamp: 'ls 7/5/1998 02:14'!
tagName
	^'input'! !

!HtmlInput methodsFor: 'attributes' stamp: 'ls 7/23/1998 19:09'!
type
	^(self getAttribute: 'type' default: 'text') asLowercase! !

HtmlEntityWithNoEndTag subclass: #HtmlMeta
	instanceVariableNames: 'theTag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlMeta commentStamp: '<historical>' prior: 0!
some result of a meta tag; unimplemented so far!

!HtmlMeta methodsFor: 'formatting' stamp: 'ls 8/12/1998 05:44'!
addToFormatter: formatter
	| httpEquiv |
	httpEquiv _ self getAttribute: 'http-equiv'.
	httpEquiv ifNil: [ ^self ].
	httpEquiv asLowercase = 'refresh' ifTrue: [
		formatter addString: '{refresh: ', (self getAttribute:  'content' default: ''), '}' ].! !

!HtmlMeta methodsFor: '*scamperm' stamp: 'ich. 1/7/2006 22:01'!
charset
	((self getAttribute: #'http-equiv') isNil or:
		[(self getAttribute: #'http-equiv') asLowercase ~= 'content-type'])
			ifTrue: [^ nil].
	((self getAttribute: #content) findTokens: '; ') do:
		[:each |
		(each beginsWith: 'charset') ifTrue:
			[^ (each findTokens: '= ') at: 2]].
	^ nil.
! !

!HtmlMeta methodsFor: 'initialization' stamp: 'ls 7/5/1998 02:23'!
initialize: aTag
	super initialize: aTag.
	theTag _ aTag.! !

!HtmlMeta methodsFor: 'as yet unclassified' stamp: 'ich 1/16/2006 01:09'!
isHeadElement
	^true! !

!HtmlMeta methodsFor: 'testing' stamp: 'ls 7/5/1998 02:22'!
mayContain: anEntity
	^false! !

!HtmlMeta methodsFor: 'printing' stamp: 'ls 7/5/1998 02:24'!
printOn: aStream indent: indent
	indent timesRepeat: [ aStream space ].
	aStream nextPutAll: 'meta: '.
	theTag printOn: aStream.
	aStream cr.! !

!HtmlMeta methodsFor: 'testing' stamp: 'ls 7/5/1998 02:22'!
tagName
	^'meta'! !

!HtmlEntity methodsFor: '*scamperm' stamp: 'ich. 5/24/2004 09:31'!
asHtmlAfterScripts
	| aStream |
	aStream := WriteStream on: ''.
	self doScriptOn: aStream indent: 0.
	^aStream contents.
! !

!HtmlEntity methodsFor: '*scamperm' stamp: 'ich. 1/8/2006 20:15'!
detect: aBooleanBlock
	(aBooleanBlock value: self) ifTrue: [^self].
	self contents isEmpty ifFalse: [^ self contents detect: aBooleanBlock].
	^nil.
! !

!HtmlEntity methodsFor: '*scamperm' stamp: 'ich. 5/24/2004 09:22'!
doScriptOn: aStream indent: indent
	aStream next: indent put: $ ;
	nextPutAll: '<';
	 nextPutAll: self tagName.
	self attributes associationsDo: [:assoc | aStream space; nextPutAll: assoc key; nextPutAll: '="'; nextPutAll: assoc value; nextPutAll: '"'].
	aStream nextPut: $>;
	 cr.
	contents do: [:entity | entity doScriptOn: aStream indent: indent + 1].
	aStream nextPutAll: '</'; nextPutAll: self tagName; nextPutAll: '>'.
! !

HtmlEntity subclass: #HtmlHtml
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlHtml class methodsFor: 'instance creation' stamp: 'ich 1/15/2006 18:22'!
emptyDocument
	"return an empty document"
	^super new add: HtmlHead new; add: HtmlBody new! !

!HtmlHtml methodsFor: 'access' stamp: 'ich 1/15/2006 18:22'!
addToBody: anObject
	"add an object to the Body entity of the receiver"
	self body add: anObject! !

!HtmlHtml methodsFor: 'access' stamp: 'ich 1/15/2006 18:22'!
addToHead: anObject
	"add an object to the head entity of the receiver"
	self head add: anObject! !

!HtmlHtml methodsFor: 'access' stamp: 'ich 1/15/2006 18:24'!
body
	^self contents detect: [:each | each isMemberOf: HtmlBody] ifNone: [nil]! !

!HtmlHtml methodsFor: '*scamperm' stamp: 'ich 1/15/2006 18:22'!
charset
	^ self head charset.
! !

!HtmlHtml methodsFor: 'formatting' stamp: 'ich 1/15/2006 18:22'!
formattedText
	"return a version of this document as a formatted Text"
	| formatter |
	formatter _ HtmlFormatter preferredFormatterClass new.
	self addToFormatter: formatter.
	^formatter text ! !

!HtmlHtml methodsFor: 'formatting' stamp: 'ich 1/15/2006 18:22'!
formattedTextForBrowser: browser  defaultBaseUrl: defaultBaseUrl
	"return a version of this document as a formatted Text (which includes links and such)"
	| formatter text |

	"set up the formatter"
	formatter _ HtmlFormatter preferredFormatterClass new.
	formatter browser: browser.
	formatter baseUrl: defaultBaseUrl.  "should check if the document specifies something else"

	"do the formatting"
	self addToFormatter: formatter.

	"get and return the result"
	text _ formatter text.
	^text! !

!HtmlHtml methodsFor: 'formatting' stamp: 'ich 1/15/2006 18:22'!
formattedTextMorph
	"return a version of this document as a formatted TextMorph (which includes links and such)"
	| formatter text textMorph |
	formatter _ HtmlFormatter preferredFormatterClass new.
	self addToFormatter: formatter.
	text _ formatter text .

	textMorph _ TextMorph new initialize.
	textMorph contentsWrapped: text.

	^textMorph! !

!HtmlHtml methodsFor: 'formatting' stamp: 'ich 1/15/2006 18:22'!
formattedTextMorphForBrowser: browser  defaultBaseUrl: defaultBaseUrl
	"return a version of this document as a formatted TextMorph (which includes links and such)"
	| formatter textMorph |

	"set up the formatter"
	formatter _ HtmlFormatter preferredFormatterClass new.
	formatter browser: browser.
	formatter baseUrl: defaultBaseUrl.  "should check if the document specifies something else"

	"do the formatting"
	self addToFormatter: formatter.

	"get and return the result"
	textMorph _ formatter textMorph .
	^textMorph! !

!HtmlHtml methodsFor: 'access' stamp: 'ich 1/15/2006 18:25'!
head
	^self contents detect: [:each | each isMemberOf: HtmlHead] ifNone: [nil]! !

!HtmlHtml methodsFor: 'testing' stamp: 'ich 1/15/2006 18:22'!
mayContain: anElement
	^true 	"not strictly true, but it makes the parser simpler"! !

!HtmlHtml methodsFor: '*scamperm' stamp: 'ich 1/15/2006 18:22'!
replaceAllTextsToWideString: aConverter 
	self allSubentitiesDo: 
			[:each | 
			(each isMemberOf: HtmlTextEntity) | (each isMemberOf: HtmlInput) 
				ifTrue: [each replaceToWideString: aConverter]]! !

!HtmlHtml methodsFor: 'testing' stamp: 'ich 1/15/2006 18:22'!
tagName
	^'html'! !

HtmlEntity subclass: #HtmlNoScript
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlNoScript methodsFor: 'as yet unclassified' stamp: 'ich 1/17/2006 22:44'!
mayContain: anEntity
	"temporarily"
	^ true
! !

!HtmlNoScript methodsFor: 'as yet unclassified' stamp: 'ich 1/17/2006 22:41'!
tagName
	^ 'noscript'! !

HtmlEntity subclass: #HtmlNotSupportedYet
	instanceVariableNames: 'tagName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlNotSupportedYet methodsFor: 'as yet unclassified' stamp: 'ich 1/16/2006 01:40'!
isHeadElement
	^ true.
! !

!HtmlNotSupportedYet methodsFor: 'as yet unclassified' stamp: 'ich 1/16/2006 01:17'!
mayContain: anEntity
	^ true.
! !

!HtmlNotSupportedYet methodsFor: 'accessing' stamp: 'ich 1/16/2006 01:14'!
tagName
	^tagName! !

!HtmlNotSupportedYet methodsFor: 'accessing' stamp: 'ich 1/16/2006 23:30'!
tagName: anObject
	tagName := anObject.
	"Transcript show: self class name , tagName , String cr."
	^ self.
! !

HtmlEntity subclass: #HtmlScript
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlScript methodsFor: 'formatting' stamp: 'ich. 9/25/2005 20:41'!
addToFormatter: formatter
! !

!HtmlScript methodsFor: 'scripting' stamp: 'ich 1/16/2006 23:16'!
doIt
	self script isEmpty ifTrue: [^''].
	(self 
		confirm: 'Do you want to execute ScamperScript on this page?' translated) 
			ifTrue: [^Compiler evaluate: self script]
			ifFalse: [^'']! !

!HtmlScript methodsFor: 'scripting' stamp: 'ich. 5/24/2004 09:26'!
doScriptOn: aStream indent: indent
	aStream next: indent put: $ ;
		nextPutAll: self doIt asString;
		cr.
! !

!HtmlScript methodsFor: 'testing' stamp: 'ich. 9/25/2005 20:23'!
isHeadElement
	"script tag may be in header."
	^true
! !

!HtmlScript methodsFor: 'testing' stamp: 'ich. 9/25/2005 20:34'!
mayContain: anEntity
	^anEntity isComment | anEntity isTextualEntity.! !

!HtmlScript methodsFor: 'scripting' stamp: 'ich. 1/9/2006 00:35'!
script
	| script |
	(self getAttribute: #type) = 'text/smalltalk' ifFalse: [^''].
	script := ''.
	self subEntities do: 
			[:each | 
			| startIndex endIndex |
			startIndex := 1 + (each commentText beginsWith: '<!!--') ifTrue: ['<!!--' size] ifFalse: [0].
			endIndex := each commentText size - (each commentText endsWith: '//-->') ifTrue: ['//-->' size] ifFalse: [0].
			script := script , (each commentText copyFrom: startIndex to: endIndex)].
	^script! !

!HtmlScript methodsFor: 'testing' stamp: 'ich. 5/23/2004 22:53'!
tagName
	^'script'.
! !

HtmlEntity subclass: #HtmlStyle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlStyle methodsFor: 'formatting' stamp: 'ich 1/17/2006 23:38'!
addToFormatter: formatter
	"head elements are normally just meta-information, and thus don't add anything to the formatter"! !

!HtmlStyle methodsFor: 'testing' stamp: 'ich 1/16/2006 01:08'!
isHeadElement
	^true! !

!HtmlStyle methodsFor: 'testing' stamp: 'ls 7/5/1998 00:15'!
mayContain: anEntity
	^anEntity isTextualEntity! !

!HtmlStyle methodsFor: 'testing' stamp: 'ls 7/5/1998 00:15'!
tagName
	^'style'! !

HtmlEntity subclass: #HtmlTitle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM-Parser Entities'!

!HtmlTitle methodsFor: 'formatting' stamp: 'ich 1/17/2006 23:38'!
addToFormatter: formatter
	"head elements are normally just meta-information, and thus don't add anything to the formatter"! !

!HtmlTitle methodsFor: 'testing' stamp: 'ls 6/27/1998 15:35'!
isHeadElement
	^true! !

!HtmlTitle methodsFor: 'testing' stamp: 'ls 6/27/1998 15:35'!
mayContain: anEntity
	^anEntity isTextualEntity! !

!HtmlTitle methodsFor: 'testing' stamp: 'ls 6/27/1998 15:36'!
tagName
	^'title'! !

!BrowserUrl methodsFor: '*scamperm' stamp: 'ich. 12/23/2005 23:32'!
retrieveContentsForBrowser: aBrowser
	^aBrowser browserUrlContents: locator! !

!EUCJPTextConverter class methodsFor: '*scamperm' stamp: 'ich. 5/1/2004 16:47'!
checkCode: upperCharacter followed: lowerCharacter
	| u l |
	u _ upperCharacter asInteger.
	u < 16r80 ifTrue: [^ true].
	l _ lowerCharacter asInteger.
	(u = 16r8E) & (l between: 16rA0 and: 16rDF) ifTrue: [^ true]. "hankaku katakana"
	(u between: 16rA1 and: 16rFE) & (l between: 16rA1 and: 16rFE) ifTrue: [^ true].
	^ false.


! !

!EUCJPTextConverter class methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 22:20'!
encodingNames
	"I found the site that returns eucJP-win as charset, so I added it. If acceptable, this method should return to the method category printing. If acceptable, this method should return to the method category utilities.
	ich. 01/05/06"
	^#('euc-jp' 'eucjp' 'eucjp-win' ) copy! !

!EUCJPTextConverter class methodsFor: '*scamperm' stamp: 'ich. 5/1/2004 18:07'!
isFirstByte: aCharacter
	^ (aCharacter asInteger = 16r8E) | (aCharacter asInteger between: 16rA1 and: 16rFE).
! !

!HtmlDocument methodsFor: '*scamperm' stamp: 'ich 1/15/2006 18:30'!
body
	^ (self contents detect: [:each | each isMemberOf: HtmlHtml]) body.
! !

!HtmlDocument methodsFor: '*scamperm' stamp: 'ich. 9/25/2005 23:08'!
charset
	^ self head charset.
! !

!HtmlDocument methodsFor: '*scamperm' stamp: 'ich 1/16/2006 01:20'!
doScriptOn: aStream indent: indent
	contents do: [:entity | entity doScriptOn: aStream indent: indent].
! !

!HtmlDocument methodsFor: '*scamperm' stamp: 'ich 1/15/2006 18:30'!
head
	^ (self contents detect: [:each | each isMemberOf: HtmlHtml]) head.
! !

!HtmlDocument methodsFor: '*scamperm' stamp: 'ich 1/15/2006 18:28'!
mayContain: anElement
	^ (anElement isMemberOf: HtmlCommentEntity) | (anElement isMemberOf: HtmlHtml).
! !

!HtmlDocument methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 22:37'!
replaceAllTextsToWideString: aConverter 
	self allSubentitiesDo: 
			[:each | 
			(each isMemberOf: HtmlTextEntity) | (each isMemberOf: HtmlInput) 
				ifTrue: [each replaceToWideString: aConverter]]! !

!HtmlDocument methodsFor: '*scamperm' stamp: 'ich 1/15/2006 18:28'!
tagName
	^''! !

!FlashMorphReader class methodsFor: '*scamperm' stamp: 'ich. 12/23/2005 21:51'!
play: source
	| stream player |
	stream := (RWBinaryOrTextStream with: source content) binary reset.
	(FlashFileReader canRead: stream) ifFalse:[^nil]. "Not a flash file"
	player := (FlashMorphReader on: stream) processFileAsync.
	player sourceUrl: source url.
	player startPlaying.
	^ player
! !

!ByteString methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 23:04'!
encodeForHTTP
	"change dangerous characters to their %XX form, for use in HTTP transactions"
	| encodedStream |
	encodedStream := WriteStream on: (String new).
	
	self do: [ :c |
		c isSafeForHTTP ifTrue: [ encodedStream nextPut: c ] ifFalse: [
			encodedStream nextPut: $%.
			encodedStream nextPut: (c asciiValue // 16) asHexDigit.
			encodedStream nextPut: (c asciiValue \\ 16) asHexDigit.
		]
	].
	^encodedStream contents. ! !

!HtmlTextEntity methodsFor: '*scamperm' stamp: 'ich. 5/24/2004 10:33'!
doScriptOn: aStream indent: indent 
	self printHtmlOn: aStream indent: indent.
! !

!HtmlTextEntity methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 22:50'!
printHtmlOn: aStream indent: indent 
	"I cared character which should be escaped in html. If acceptable, this method should return to the method category printing.
	ich. 01/05/06"

	text do: 
			[:each | 
			each = $< 
				ifTrue: [aStream nextPutAll: '&lt;']
				ifFalse: 
					[each = $> 
						ifTrue: [aStream nextPutAll: '&gt;']
						ifFalse: [aStream nextPut: each]]]! !

!HtmlTextEntity methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 22:37'!
replaceToWideString: aTextConverter 
	aTextConverter 
		ifNotNil: [self text: (self text convertFromWithConverter: aTextConverter)]! !

!ServerDirectory methodsFor: '*scamperm' stamp: 'ich. 5/2/2004 00:11'!
readOnlyFileNamed: aName
	"If the file exists, answer a read-only RemoteFileStream on it.  aName is directory path, and does include name of the server.  Or it can just be a fileName.  For now, pre-read the file."

	| rFile |

	rFile _ self asServerFileNamed: aName.
	rFile readOnly.
	rFile isTypeFile ifTrue: [
		^ FileStream oldFileNamed: (rFile fileNameRelativeTo: self)].

	^ rFile streamOnBeginning! !

!ServerFile methodsFor: '*scamperm' stamp: 'ich. 5/2/2004 00:21'!
downloadUrl
	"The url under which files will be accessible."
	^ self altUrl
		ifNil: [self realUrl]
		ifNotNil: [self altUrl].

! !

!ServerFile methodsFor: '*scamperm' stamp: 'ich. 5/2/2004 00:15'!
getInto: dataStream
	self getFileNamed: '' into: dataStream.
! !

!ServerFile methodsFor: '*scamperm' stamp: 'ich. 5/2/2004 00:28'!
streamOnBeginning
	| remoteStrm |
	remoteStrm _ RemoteFileStream on: (String new: 2000).
	remoteStrm remoteFile: self.
	self getInto: remoteStrm.	"prefetch data"
	^ remoteStrm! !

!HierarchicalUrl methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 22:14'!
privateParsePath: remainder relativeTo: basePath 
	"The method unescapePercents changes URL encoded Japanese string to WideString. It doesn't work for retriving, so I commented out. If acceptable, this method should return to the method category parsing.
	ich. 01/05/06."

	| nextTok s parsedPath |
	s := ReadStream on: remainder.
	parsedPath := OrderedCollection new.
	parsedPath addAll: basePath.
	parsedPath isEmpty ifFalse: [parsedPath removeLast].
	
	[s peek = $/ ifTrue: [s next].
	nextTok := WriteStream on: String new.
	[s atEnd or: [s peek = $/]] whileFalse: [nextTok nextPut: s next].
	nextTok := nextTok contents.	"unescapePercents"
	nextTok = '..' 
		ifTrue: [parsedPath size > 0 ifTrue: [parsedPath removeLast]]
		ifFalse: [nextTok ~= '.' ifTrue: [parsedPath add: nextTok]].
	s atEnd] 
			whileFalse.
	parsedPath isEmpty ifTrue: [parsedPath add: ''].
	^parsedPath! !

!HierarchicalUrl methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 22:16'!
toText
	"http://www.yahoo.co.jp/ uses previous searh parameters as directory path and they include character =. The method encodeForHTTP to pathElem changes = to %XX, it doesn't work for yahoo.co.jp site, so I commented out it. If acceptable, this method should return to the method category printing.
	ich. 01/05/06"

	| ans |
	ans := WriteStream on: String new.
	ans nextPutAll: self schemeName.
	ans nextPutAll: '://'.
	self username ifNotNil: 
			[ans nextPutAll: self username.
			self password ifNotNil: 
					[ans nextPutAll: ':'.
					ans nextPutAll: self password].
			ans nextPutAll: '@'].
	ans nextPutAll: self authority.
	port ifNotNil: 
			[ans
				nextPut: $:;
				print: port].
	path do: 
			[:pathElem | 
			ans nextPut: $/.
			ans nextPutAll: pathElem	"encodeForHTTP"].
	self query isNil 
		ifFalse: 
			[ans nextPut: $?.
			ans nextPutAll: self query].
	self fragment isNil 
		ifFalse: 
			[ans nextPut: $#.
			ans nextPutAll: self fragment encodeForHTTP].
	^ans contents! !

!String methodsFor: '*scamperm' stamp: 'ich. 1/5/2006 23:06'!
encodeForHTTP
	self subclassResponsibility
! !

!String methodsFor: '*scamperm' stamp: 'ich. 9/25/2005 22:45'!
guessJapaneseEncoding
	| stream eucCount sjisCount eucFirstByte sjisFirstByte |
	self mayBeJIS ifTrue: [^ 'iso-2022-jp'].
		
	stream _ ReadStream on: self.
	eucCount _ 0.
	sjisCount _ 0.
	eucFirstByte _ nil.
	sjisFirstByte _ nil.
	[stream atEnd] whileFalse:
		[| char |
		char _ stream next.
		eucFirstByte
			ifNil:
				[(EUCJPTextConverter isFirstByte: char) ifTrue:
					[eucFirstByte _ char]]
			ifNotNil:
				[(EUCJPTextConverter checkCode: eucFirstByte followed: char) ifTrue:
					[eucCount _ eucCount + 1].
				eucFirstByte _ nil].
		sjisFirstByte
			ifNil:
				[(ShiftJISTextConverter isFirstByte: char) ifTrue:
					[sjisFirstByte _ char]]
			ifNotNil:
				[(ShiftJISTextConverter checkCode: sjisFirstByte followed: char) ifTrue:
					[sjisCount _ sjisCount + 1].
				sjisFirstByte _ nil]].

	(eucCount = 0) & (sjisCount = 0) ifTrue: [^ nil]. "It means ASCII."
	eucCount >= sjisCount ifTrue: [^ 'eucjp']. "EUC has high priority."
	^ 'sjis'.
! !

!String methodsFor: '*scamperm' stamp: 'ich. 5/8/2004 16:22'!
mayBeJIS
	| esc |
	esc _ Character escape asString.
	^ (self includesSubString: esc , '$@') or:
		[(self includesSubString: esc , '$B') or:
			[self includesSubString: esc , '$(D']].
! !

Model subclass: #ScamperM
	instanceVariableNames: 'webRetriever'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScamperM'!

!ScamperM commentStamp: '<historical>' prior: 0!
a multilingual web browser.

A custom start page is provided for.  It's not as convenient as bookmarks are, but it does have a lot of flexibility--the user can put anything on the start page that is desired.
!

!ScamperM class methodsFor: 'private' stamp: 'ich. 12/28/2005 00:11'!
defaultUrl
	^ 'browser:about'.
! !

!ScamperM class methodsFor: 'fileIn/Out' stamp: 'ich. 12/28/2005 01:09'!
fileReaderServicesForFile: fullName suffix: suffix
	^ (#('htm' 'html') includes: suffix) 
		ifTrue: [self services]
		ifFalse: [#()]! !

!ScamperM class methodsFor: 'class initialization' stamp: 'ich. 1/5/2006 23:22'!
initialize
	FileList registerFileReader: self.
	self registerInOpenMenu.
	WebBrowser register: self.
	Flaps registerQuad: {
				#ScamperM.
				#newOpenableMorph.
				(self asString).
				'A multilingual web browser'
			}
		forFlapNamed: 'Tools'! !

!ScamperM class methodsFor: 'initialize-release' stamp: 'ich. 1/3/2006 16:33'!
newInstancePointingAt: aStringOrUrl
	^ (self new
		jumpToAbsoluteUrl: aStringOrUrl asUrl) morphicWindow applyModelExtent! !

!ScamperM class methodsFor: 'initialize-release' stamp: 'ich. 12/28/2005 01:08'!
newOpenableMorph
	"Answer a morph that bears the receiver, but don't blast it onto the screen"

	^ self newInstancePointingAt: self defaultUrl.! !

!ScamperM class methodsFor: 'initialize-release' stamp: 'ich. 12/28/2005 01:09'!
newOrExistingOn: aStringOrUrl
	| siteStr |
	"If a Scamper is open on the same site, return its SystemWindow, else return a new Scamper."

	siteStr := aStringOrUrl asUrl schemeName, '://', aStringOrUrl asUrl authority.
	Smalltalk isMorphic ifTrue:
		[World submorphsDo:
			[:m | 
			((m isKindOf: SystemWindow) and: [m model isKindOf: self]) ifTrue:
				[(m model currentUrl asString beginsWith: siteStr) ifTrue:
						[^ m expand.]]]].

	^ self new open! !

!ScamperM class methodsFor: 'initialize-release' stamp: 'ich. 12/28/2005 00:01'!
open
	^ self openOnUrl: self defaultUrl.! !

!ScamperM class methodsFor: 'private' stamp: 'ich. 12/28/2005 01:09'!
openAsMorph
	"Deprecated. Use open instead."

	^ self open! !

!ScamperM class methodsFor: 'initialize-release' stamp: 'ich. 12/28/2005 01:09'!
openFileFromFileList: fullName
	self openFile: fullName! !

!ScamperM class methodsFor: 'initialize-release' stamp: 'ich. 12/28/2005 01:09'!
openFile: fullFileName
	self openOnUrl: 
		(FileDirectory forFileName: fullFileName) url , 
		(FileDirectory localNameFor: fullFileName) encodeForHTTP
! !

!ScamperM class methodsFor: 'initialize-release' stamp: 'ich. 1/3/2006 16:33'!
openOnUrl: aStringOrUrl
	^ self new
		jumpToAbsoluteUrl: aStringOrUrl asUrl;
		openAsMorph! !

!ScamperM class methodsFor: 'class initialization' stamp: 'ich. 1/5/2006 23:28'!
registerInOpenMenu
	TheWorldMenu registerOpenCommand: {
				'Multilingual Web Browser'.
				{ self. #open }.
				('"' , self asString , '", a multilingual web browser')
			}! !

!ScamperM class methodsFor: 'fileIn/Out' stamp: 'ich. 12/28/2005 01:09'!
serviceOpenInWebBrowser
	"Answer a service for opening a web browser on a file"

	^ SimpleServiceEntry 
			provider: self 
			label: 'open in web browser'
			selector: #openFile:
			description: 'open in web browser'
			buttonLabel: 'open'! !

!ScamperM class methodsFor: 'fileIn/Out' stamp: 'ich. 12/28/2005 01:09'!
services

	^ Array with: self serviceOpenInWebBrowser

! !

!ScamperM class methodsFor: 'class initialization' stamp: 'ich. 1/3/2006 17:05'!
unload
	FileList unregisterFileReader: self.
	TheWorldMenu unregisterOpenCommandWithReceiver: self.
	Flaps unregisterQuadsWithReceiver: self.
	WebBrowser unregister: self.! !

!ScamperM methodsFor: 'private-browser urls' stamp: 'ich. 1/7/2006 22:52'!
aboutScamperHTML
	"return a string of HTML which introduces Scamper"
	^'
<html>
<head>
<title>{1}</title>
</head>
<body>
<h1>{2}</h1>
{3}
<ul>
<li><a href="http://minnow.cc.gatech.edu/squeak">{4}</a>
<li><a href="http://minnow.cc.gatech.edu/squeak/14">{5}</a>
<li><a href="http://www.squeak.org">{6}</a>
<li><a href="http://www.squeakland.org">{7}</a>
<li><a href="http://ich.homelinux.net:8080/squeak/58">{8}</a>
<li><a href="http://languagegame.org:8080/propella/91">{9}</a>
</ul>
</body>
</html>' format:{
	'About ScamperM' translated.
	'ScamperM' translated.
	'This is ScamperM, a multilingual web browser for Squeak.  Here are some URL''s to start at:' translated.
	'The Squeak Swiki' translated.
	'Scamper''s Home Page' translated.
	'Squeak''s Home Page' translated.
	'The SqueakLand Home Page' translated.
	'ScamperM''s Home Page' translated.
	'ScamperWorkspace''s Home Page' translated.
	}! !

!ScamperM methodsFor: 'private-button text' stamp: 'ich. 12/28/2005 00:22'!
backButtonText
	^ 'Go back to previous URL in history' translated! !

!ScamperM methodsFor: 'private-browser urls' stamp: 'ich. 1/3/2006 16:03'!
browserUrlContents: aRequest
	aRequest = 'start' ifTrue:
		[^MIMEDocument contentType: 'text/html' content: self startPage].
	aRequest = 'about' ifTrue:
		[^MIMEDocument contentType: 'text/html' content: self aboutScamperHTML].
	^ nil! !

!ScamperM methodsFor: 'private-user interface' stamp: 'ich. 1/11/2006 01:00'!
buttonRowPane
	"Create and return a pane of navigation buttons."

	| buttonRow |
	buttonRow := (AlignmentMorph new)
				borderWidth: 0;
				layoutInset: 0;
				hResizing: #spaceFill;
				wrapCentering: #center;
				cellPositioning: #leftCenter;
				clipSubmorphs: true;
				addTransparentSpacerOfSize: 5 @ 0;
				yourself.
	buttonRow
		addMorphBack: (self 
					simpleButtonNamed: 'Back' translated
					action: #back
					text: self backButtonText);
		addTransparentSpacerOfSize: 5 @ 0;
		addMorphBack: (self 
					simpleButtonNamed: 'Forward' translated
					action: #forward
					text: self forwardButtonText);
		addTransparentSpacerOfSize: 5 @ 0;
		addMorphBack: (self 
					simpleButtonNamed: 'History' translated
					action: #displayHistory
					text: self historyButtonText);
		addTransparentSpacerOfSize: 5 @ 0;
		addMorphBack: (self 
					simpleButtonNamed: 'Reload' translated
					action: #reload
					text: self reloadButtonText);
		addTransparentSpacerOfSize: 5 @ 0;
		addMorphBack: (self 
					simpleButtonNamed: 'Home' translated
					action: #visitStartPage
					text: self homeButtonText);
		addTransparentSpacerOfSize: 5 @ 0;
		addMorphBack: (self 
					simpleButtonNamed: 'Stop!!' translated
					action: #stopEverything
					text: self stopButtonText);
		addTransparentSpacerOfSize: 5 @ 0;
		addMorphBack: (StringMorph contents: 'Workers:');
		addTransparentSpacerOfSize: 5 @ 0;
		addMorphBack: webRetriever workerCountEdit.
	^buttonRow! !

!ScamperM methodsFor: 'accessing' stamp: 'ich. 1/3/2006 16:38'!
currentUrl
	^ webRetriever currentUrl.
! !

!ScamperM methodsFor: 'private-button text' stamp: 'ich. 12/28/2005 00:22'!
forwardButtonText
	^ 'Go forward to next URL in history' translated! !

!ScamperM methodsFor: 'private-button text' stamp: 'ich. 12/28/2005 00:23'!
historyButtonText
	^ 'Return to a recent URL in history' translated! !

!ScamperM methodsFor: 'private-button text' stamp: 'ich. 12/28/2005 00:24'!
homeButtonText
	^ 'Go to start page' translated! !

!ScamperM methodsFor: 'initialize-release' stamp: 'ich. 12/29/2005 23:46'!
initialize
	webRetriever := WebRetriever new.
	webRetriever addDependent: self.

! !

!ScamperM methodsFor: 'testing' stamp: 'ich. 1/3/2006 16:57'!
isWebBrowser
	^true! !

!ScamperM methodsFor: 'changing page' stamp: 'ich. 1/3/2006 16:32'!
jumpToAbsoluteUrl: urlText
	webRetriever jumpToAbsoluteUrl: urlText.
! !

!ScamperM methodsFor: 'initialize-release' stamp: 'ich. 1/8/2006 11:20'!
labelString
	^self class asString , (webRetriever htmlDocument isNil 
				ifTrue: ['']
				ifFalse: 
					[': ' , (webRetriever htmlDocument head title ifNil: ['(untitled)' translated])])! !

!ScamperM methodsFor: 'private-user interface' stamp: 'ich. 1/5/2006 20:57'!
morphicWindow
	^(SystemWindow labelled: self class asString)
		model: self;
		setProperty: #webBrowserView toValue: true;
		addMorph: self buttonRowPane frame: (0 @ 0 extent: 1 @ 0.07);
		addMorph: self urlEditPane frame: (0 @ 0.07 extent: 1 @ 0.06);
		addMorph: self webContentsPane frame: (0 @ 0.13 extent: 1 @ 0.81);
		addMorph: self statusPane frame: (0 @ 0.94 extent: 1.0 @ 0.06);
		yourself
! !

!ScamperM methodsFor: 'user interface' stamp: 'ich. 12/28/2005 00:08'!
openAsMorph

	^self morphicWindow openInWorld
! !

!ScamperM methodsFor: 'initialize-release' stamp: 'ich. 1/3/2006 12:04'!
release
	webRetriever removeDependent: self.
	super release.
! !

!ScamperM methodsFor: 'private-button text' stamp: 'ich. 12/28/2005 00:24'!
reloadButtonText
	^ 'Reload page' translated! !

!ScamperM methodsFor: 'private-user interface' stamp: 'ich. 12/28/2005 00:30'!
simpleButtonNamed: buttonLabel action: selector text: balloonText
	"Create and return a simple button with Scamper as the target."

	^SimpleButtonMorph new
		label: buttonLabel;
		target: webRetriever;
		color: Color transparent;
		cornerStyle: #rounded;
		borderColor: Color black;
		actionSelector: selector;
		actWhen: #buttonUp;
		setBalloonText: balloonText;
		yourself
! !

!ScamperM methodsFor: 'private-browser urls' stamp: 'ich. 1/3/2006 16:04'!
startPage
	"return the contents of the user's personal start page"
	| file |
	file := FileStream oldFileOrNoneNamed: 'StartPage.html'.
	file 
		ifNil: [ ^'<title>{1}</title>
<h1>{1}</h1>
{2}' format:{'Personal Start Page' translated. 'This space is empty' translated} ]
		ifNotNil: [ ^file contentsOfEntireFile ]! !

!ScamperM methodsFor: 'private-browser urls' stamp: 'ich. 1/3/2006 16:04'!
startPage: newPage
	"fill in the contents of the user's personal start page"

	| file |
	FileDirectory default deleteFileNamed: 'StartPage.html'.
	[file := FileStream fileNamed: 'StartPage.html'.
	file ifNil: [self error: 'could not save file' translated].
	file nextPutAll: newPage asString.] ensure: [file close].
	self changed: #startPage.
	^true! !

!ScamperM methodsFor: 'private-user interface' stamp: 'ich. 12/28/2005 00:25'!
statusPane
	^ (PluggableTextMorph
		on: webRetriever
		text: #status
		accept: nil)
		hideScrollBarsIndefinitely;
		yourself.
! !

!ScamperM methodsFor: 'private-button text' stamp: 'ich. 12/28/2005 00:24'!
stopButtonText
	^ 'Stop loading page' translated! !

!ScamperM methodsFor: 'updating' stamp: 'ich. 1/3/2006 16:40'!
update: anAspect
	anAspect = #currentUrl ifTrue:
		[self changed: #currentUrl;
			changed: #relabel].
! !

!ScamperM methodsFor: 'private-user interface' stamp: 'ich. 1/3/2006 16:38'!
urlEditPane
	"Create and return the URL edit pane."

	| pane |
	pane := (PluggableTextMorph on: self text: #currentUrl accept: #jumpToAbsoluteUrl:).
	pane acceptOnCR: true.
	pane hideScrollBarsIndefinitely.
	^ pane 
		! !

!ScamperM methodsFor: 'private-user interface' stamp: 'ich. 12/28/2005 00:25'!
webContentsPane
	"Create and return the web page pane."

	^WebPageMorph
		on: webRetriever 
		bg: #backgroundColor 
		text: #formattedPage 
		readSelection: #formattedPageSelection 
		menu: #menu:shifted:! !

Model subclass: #WebRetriever
	instanceVariableNames: 'status currentUrl pageSource formattedPage downloadingProcess documentQueue currentAnchorLocation backgroundColor history bindings htmlDocument workerCountEdit'
	classVariableNames: 'Favorite'
	poolDictionaries: ''
	category: 'ScamperM'!

!WebRetriever commentStamp: '<historical>' prior: 0!
a web rendering engine.

It downloads things in a background thread; the background thread puts downloaded objects onto downloadQueue as they arrive.  The queue is checked in the step method.

There is a hook for displaying the "lint" of a page; currently it's not a very effective linter.

You can doIt within web page (ScamperWorkspace).!

!WebRetriever class methodsFor: 'class initialization' stamp: 'ich. 12/28/2005 23:59'!
initialize
	Favorite := Bookmark new.
! !

!WebRetriever methodsFor: 'access' stamp: 'ich. 12/28/2005 23:58'!
addToBookmark
	| title |
	title := self document head title ifNil: ['Untitled' translated].
	Favorite save: title,'.lin' forLinkOf: self currentUrl.
	Favorite add: (Association key: title value: self currentUrl).
! !

!WebRetriever methodsFor: 'menus' stamp: 'ich. 1/4/2006 20:58'!
back
	"The user hit the back button -- go to the previous document"

	| doc |
	doc := history back.
	doc ifNil: [^self].
	self storeAndDisplay: doc! !

!WebRetriever methodsFor: 'accessing' stamp: 'bolot 2/27/2000 20:59'!
backgroundColor
	^backgroundColor ifNil:
		[self defaultBackgroundColor]! !

!WebRetriever methodsFor: 'bindings' stamp: 'ich. 1/7/2006 22:08'!
bindingOf: aString 
	bindings isNil ifTrue: [bindings := Dictionary new].
	(bindings includesKey: aString) ifFalse: [bindings at: aString put: nil].
	^bindings associationAt: aString! !

!WebRetriever methodsFor: 'menus' stamp: 'ich. 12/28/2005 23:58'!
bookmark
	Favorite showMenuFor: self.
! !

!WebRetriever methodsFor: 'change/update' stamp: 'ich. 12/29/2005 23:56'!
changedAll: aspects 
	aspects do: [:symbol | self changed: symbol]! !

!WebRetriever methodsFor: 'accessing' stamp: 'ich. 1/8/2006 12:17'!
charsetOf: inputs
	(inputs includesKey: 'ie') 
		ifTrue: [^(inputs at: 'ie') first].
	(inputs includesKey: 'ei')
		ifTrue: [^(inputs at: 'ei') first].
	^nil.
! !

!WebRetriever methodsFor: 'testing' stamp: 'ich. 1/8/2006 12:00'!
checkFormUrl: url andMethod: method 
	(url asUrlRelativeTo: currentUrl) schemeName ~= 'http' 
		ifTrue: 
			[self inform: 'I can only submit forms via HTTP' translated.
			^false].
	(#('get' 'post' ) includes: method asLowercase) 
		ifFalse: 
			[self inform: ('unknown FORM method: {1}' translated format: { method }).
			^false].
	^true! !

!WebRetriever methodsFor: 'accessing' stamp: 'ich 1/21/2006 11:38'!
currentCharset
	| charset |
	htmlDocument ifNotNil:
		[charset := htmlDocument charset].
	charset isNil & (LocaleID current isoLanguage = 'ja') ifTrue:
		[charset := pageSource guessJapaneseEncoding].
	^charset! !

!WebRetriever methodsFor: 'accessing' stamp: 'ls 7/14/1998 03:18'!
currentUrl
	^currentUrl! !

!WebRetriever methodsFor: 'private' stamp: 'bolot 2/27/2000 20:59'!
defaultBackgroundColor
	^Color white! !

!WebRetriever methodsFor: 'private' stamp: 'ich. 1/11/2006 00:57'!
defaultWorkerCount
	^5! !

!WebRetriever methodsFor: 'private-document handling' stamp: 'ich. 1/3/2006 18:22'!
displayFlashPage: mimeDocument 
	| player |
	player := FlashMorphReader play: mimeDocument.
	player ifNil: [^false].
	formattedPage := (Character value: 1) asText.
	backgroundColor := self defaultBackgroundColor.
	formattedPage 
		addAttribute: (TextAnchor new anchoredMorph: player)
		from: 2
		to: 2.
	self setCurrentPage: mimeDocument.
	^true! !

!WebRetriever methodsFor: 'menus' stamp: 'ich. 1/4/2006 20:58'!
displayHistory
	history menuFor: self selector: #storeAndDisplay:! !

!WebRetriever methodsFor: 'private-document handling' stamp: 'ich. 1/3/2006 18:23'!
displayImagePage: mimeDocument 
	| handled |
	handled := true.
	backgroundColor := self defaultBackgroundColor.
	formattedPage := 
			[| imageMorph |
			imageMorph := ImageMorph new image: (ImageReadWriter 
								formFromStream: (RWBinaryOrTextStream with: mimeDocument content) binary 
										reset).
			(Character value: 1) asText 
				addAttribute: (TextAnchor new anchoredMorph: imageMorph)
				from: 2
				to: 2] 
					ifError: [:msg :ctx | handled := false].
	self setCurrentPage: mimeDocument.
	^handled! !

!WebRetriever methodsFor: 'private-document handling' stamp: 'ich. 1/9/2006 00:20'!
displayPlainTextPage: mimeDocument 
	"treat as plain text"

	pageSource := mimeDocument content.
	htmlDocument := nil.
	formattedPage := (pageSource convertFromWithConverter:
			(TextConverter newForEncoding: self currentCharset)) 
				withSqueakLineEndings.
	backgroundColor := self defaultBackgroundColor.
	currentUrl := mimeDocument url.
	self status: 'sittin' translated.
	self changedAll: #(
				#currentUrl
				#hasLint
				#lint
				#formattedPage
				#formattedPage
				#formattedPageSelection
			).
	^true! !

!WebRetriever methodsFor: 'private-html handling' stamp: 'ich. 1/8/2006 20:56'!
displayTextHtmlPage: mimeDocument 
	currentUrl := mimeDocument url.
	pageSource := mimeDocument content.
	self parsePage.
	self formatPage.
	self changedAll: #(
				#currentUrl
				#hasLint
				#lint
				#backgroundColor
				#formattedPage
				#formattedPageSelection
			).
	self status: 'done.' translated.
	^true! !

!WebRetriever methodsFor: 'document handling' stamp: 'ich. 1/4/2006 20:50'!
display: aMIMEDocument
	"this method should be placed in MIMEDocument class."

	aMIMEDocument mainType = 'image' 
		ifTrue: [(self displayImagePage: aMIMEDocument) ifTrue: [^true]].
	(aMIMEDocument contentType beginsWith: 'text/html') 
		ifTrue: [(self displayTextHtmlPage: aMIMEDocument) ifTrue: [^true]].
	aMIMEDocument contentType = 'x-application/shockwave-flash' 
		ifTrue: [(self displayFlashPage: aMIMEDocument) ifTrue: [^true]].
	(#('audio/midi' 'audio/x-midi' ) includes: aMIMEDocument contentType) 
		ifTrue: [(self processMidiPage: aMIMEDocument) ifTrue: [^true]].

	"Before we display plain text files we should look at the extension of the URL"
	aMIMEDocument mainType = 'text' 
		ifTrue: 
			[(aMIMEDocument url toText endsWithAnyOf: #('.gif' '.jpg' '.pcx' )) 
				ifTrue: [(self displayImagePage: aMIMEDocument) ifTrue: [^true]].
			(aMIMEDocument url toText endsWithAnyOf: #('.mid' '.midi' )) 
				ifTrue: [(self processMidiPage: aMIMEDocument) ifTrue: [^true]].
			(aMIMEDocument url toText endsWith: '.swf') 
				ifTrue: [(self displayFlashPage: aMIMEDocument) ifTrue: [^true]].
			self displayPlainTextPage: aMIMEDocument.
			^true].
	self processUnhandledPage: aMIMEDocument.
	^true! !

!WebRetriever methodsFor: 'do it' stamp: 'ls 9/14/1998 20:15'!
doItContext
	^nil! !

!WebRetriever methodsFor: 'do it' stamp: 'ich. 1/7/2006 22:09'!
doItReceiver
	^self
! !

!WebRetriever methodsFor: 'private-download' stamp: 'ich. 1/3/2006 16:18'!
download: newUrl 
	| newSource |
	newSource := [newUrl retrieveContentsForBrowser: self webBrowser] 
				ifError: [:msg :ctx | MIMEDocument contentType: 'text/plain' content: msg].
	newSource ifNil: 
			[newSource := MIMEDocument contentType: 'text/plain'
						content: 'Error retrieving this URL' translated].
	newSource url ifNil: 
			[newSource := MIMEDocument 
						contentType: newSource contentType
						content: newSource content
						url: newUrl].
	documentQueue nextPut: newSource! !

!WebRetriever methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:40'!
editStartPage
	| win textMorph |
	Smalltalk isMorphic ifFalse: [^ self inform: 'only works for morphic currently' translated].

	win _ SystemWindow labelled: 'edit Bookmark page' translated.
	win model: self.
	textMorph _ PluggableTextMorph on: self text: #startPage  accept: #startPage:.
	win addMorph: textMorph frame: (0@0 extent: 1@1).
	win openInWorld.
	^ true! !

!WebRetriever methodsFor: 'private-html handling' stamp: 'ich. 1/9/2006 16:35'!
formatPage
	| formatter |
	self status: 'laying out...' translated.
	formatter := (DHtmlFormatter preferredFormatterClass new)
				browser: self;
				baseUrl: currentUrl;
				yourself.
	htmlDocument addToFormatter: formatter.
	formattedPage := formatter text.
	self setBackGround.
	self setCurrentAnchorLocationWith: formatter.
	self startDownloadingMorphState: formatter incompleteMorphs! !

!WebRetriever methodsFor: 'accessing' stamp: 'ls 7/21/1998 01:16'!
formattedPage
	"format the current page and return it as a Text"
 	^formattedPage ifNil: [ ^Text new ].
	! !

!WebRetriever methodsFor: 'accessing' stamp: 'ls 9/10/1998 03:38'!
formattedPageSelection
	currentAnchorLocation ifNil: [ ^0 to: -1 ].
	^currentAnchorLocation to: currentAnchorLocation! !

!WebRetriever methodsFor: 'menus' stamp: 'ich. 1/4/2006 20:58'!
forward
	"this method is added to Scamper: Aibek 4/18/99"

	| doc |
	doc := history forward.
	doc ifNil: [^self].
	self storeAndDisplay: doc! !

!WebRetriever methodsFor: 'lint' stamp: 'ich. 1/8/2006 11:17'!
hasLint
	"whether the current page has any questionable HTML in it"

	htmlDocument ifNil: [^false].
	^htmlDocument lint ~= ''! !

!WebRetriever methodsFor: 'accessing' stamp: 'ich. 1/8/2006 11:19'!
htmlDocument
	"return the current parsed HTML document, or nil if we aren't viewing a page"

	^htmlDocument! !

!WebRetriever methodsFor: 'access' stamp: 'ich. 12/28/2005 23:58'!
importBookmark
	Favorite import.

! !

!WebRetriever methodsFor: 'initialize/release' stamp: 'ich. 1/8/2006 11:17'!
initialize
	documentQueue := SharedQueue new.
	history := WebHistory new.
	currentUrl := ''.
	pageSource := ''.
	htmlDocument := HtmlParser parse: (ReadStream on: '').
	self status: 'sittin' translated! !

!WebRetriever methodsFor: 'menus' stamp: 'ich. 1/8/2006 11:17'!
inspectParseTree
	"inspect a parse tree of the current page's source"

	htmlDocument ifNotNil: [htmlDocument inspect]! !

!WebRetriever methodsFor: 'change/update' stamp: 'ich. 12/29/2005 23:56'!
invalidateLayout
	self changedAll: #(
				#backgroundColor
				#formattedPage
				#formattedPageSelection
			)! !

!WebRetriever methodsFor: 'testing' stamp: 'ich. 1/3/2006 17:02'!
isWebBrowser
	^ true.
! !

!WebRetriever methodsFor: 'changing page' stamp: 'ich. 1/3/2006 15:07'!
jumpToAbsoluteUrl: urlText 
	"start downloading a new page.  The page source is downloaded in a background thread"

	| newUrl |
	self stopEverything.
	newUrl := urlText asUrl.
	newUrl hasContents 
		ifFalse: 
			["if it fundamentally doesn't fit the pages-and-contents model used internally"

			newUrl activate.	"spawn off an external viewer for it"
			^true].
	self startDownload: newUrl.
	history clearForNewPage.
	self start.
	^true! !

!WebRetriever methodsFor: 'changing page' stamp: 'dgd 10/28/2003 13:33'!
jumpToNewUrl
	"change to a new, user-specified page"
	| newUrl |
	newUrl _ FillInTheBlank request: 'url to visit' translated initialAnswer: currentUrl toText.
	(newUrl isNil or: [ newUrl isEmpty ]) ifTrue: [ ^self ].
	self jumpToAbsoluteUrl: newUrl! !

!WebRetriever methodsFor: 'changing page' stamp: 'ls 8/20/1998 11:22'!
jumpToUrl: urlText
	"start downloading a new page.  The page source is downloaded in a background thread"
	self jumpToAbsoluteUrl: (urlText asUrlRelativeTo: currentUrl)
! !

!WebRetriever methodsFor: 'lint' stamp: 'ich. 1/8/2006 11:17'!
lint
	"return a string describing any questionable HTML that was noticed in the current page"

	"(not currently very comprehensive)"

	htmlDocument ifNil: [^''].
	^htmlDocument lint! !

!WebRetriever methodsFor: 'menus' stamp: 'ich. 1/7/2006 22:41'!
menu: menu shifted: shifted 
	| list |
	list := #(
				#('back' #back )
				#('forward' #forward )
				#-
				#('do it (d)' #doIt )
				#('print it (p)' #printIt )
				#('inspect it (i)' #inspectIt )
				#('file it in (G)' #fileItIn )
				#('debug it' #debugIt )
				#-
				#('new URL' #jumpToNewUrl )
				#('history' #displayHistory )
				#-
				#('view source' #viewSource )
				#('inspect parse tree' #inspectParseTree )
				#-
				#('go to start page' #visitStartPage )
				#('edit start page' #editStartPage )
				#('bookmark' #bookmark )
				#('translate' #translateIt )
			).
	downloadingProcess 
		ifNotNil: [list add: #(#- #('stop downloading' #stopEverything ) )].
	list do: 
			[:each | 
			(each isKindOf: String) 
				ifFalse: [each at: 1 put: (each at: 1) translated]].
	^menu
		addList: list;
		yourself! !

!WebRetriever methodsFor: 'private' stamp: 'ich. 1/3/2006 12:38'!
morph
	"Answer the morph that holds the receiver."

	^self dependents 
		detect: [:each | (each isKindOf: WebPageMorph) and: [each model == self]]
		ifNone: [nil]! !

!WebRetriever methodsFor: 'private-user interface' stamp: 'ich. 12/24/2005 19:36'!
openFileForSave
	| fileName file |
	fileName := ''.
	[fileName := FillInTheBlank
		request: 'file to save in' translated
		initialAnswer: fileName.
	fileName isEmpty ifTrue: [^nil].
	file := FileStream fileNamed: fileName.
	file isNil] whileTrue.
	^ file.
! !

!WebRetriever methodsFor: 'private-html handling' stamp: 'ich 1/21/2006 11:26'!
parsePage
	self status: 'parsing...' translated.
	htmlDocument := HtmlParser parse: (ReadStream on: pageSource).
	htmlDocument := HtmlParser parse: htmlDocument asHtmlAfterScripts.
	htmlDocument replaceAllTextsToWideString: (TextConverter 
				newForEncoding: self currentCharset)! !

!WebRetriever methodsFor: 'menus' stamp: 'ls 7/14/1998 02:28'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!WebRetriever methodsFor: 'private-document handling' stamp: 'ich. 1/3/2006 18:23'!
processMidiPage: mimeDocument 
	Smalltalk at: #MIDIFileReader
		ifPresent: 
			[:reader | 
			reader 
				playStream: (RWBinaryOrTextStream with: mimeDocument content) reset binary.
			self status: 'sittin' translated.
			^true].
	^false! !

!WebRetriever methodsFor: 'private-document handling' stamp: 'ich. 1/3/2006 18:23'!
processUnhandledPage: mimeDocument 
	"offer to save it to a file"

	self status: 'sittin' translated.
	(mimeDocument url toText endsWith: '.pr') 
		ifTrue: 
			[(self 
				confirm: 'Looks like a Squeak project - do you want to load it as such?' 
						translated) 
					ifTrue: [^ProjectLoading thumbnailFromUrl: mimeDocument url toText]].
	(self 
		confirm: ('unkown content-type {1}--
Would you like to save to a file?' 
				translated format: { (mimeDocument contentType) })) 
			ifFalse: [^self].
	self save: mimeDocument! !

!WebRetriever methodsFor: 'private-html handling' stamp: 'ich. 1/8/2006 11:17'!
reformatPage
	"images and such have been downloaded; update the page"

	self status: 'reformatting page...' translated.
	formattedPage := htmlDocument formattedTextForBrowser: self
				defaultBaseUrl: currentUrl.
	backgroundColor := Color fromString: htmlDocument body bgcolor.
	self changedAll: #(#backgroundColor #formattedPage ).
	^self status: 'sittin' translated! !

!WebRetriever methodsFor: 'initialize/release' stamp: 'ls 7/29/1998 03:28'!
release
	self stopEverything.
	super release.! !

!WebRetriever methodsFor: 'menus' stamp: 'kfr 6/11/2000 13:24'!
reload
	self stopEverything.
	self jumpToUrl: currentUrl
	
! !

!WebRetriever methodsFor: 'private' stamp: 'ich. 1/11/2006 00:57'!
resetWorkerCount
	workerCountEdit
		contents: self defaultWorkerCount asString;
		autoFit: true;
		wrapFlag: true;
		extent: 30 @ 20! !

!WebRetriever methodsFor: 'private' stamp: 'ich. 1/9/2006 00:04'!
restoreEncodingOf: inputs 
	"assumes current document is HTML.
	changes aDictionaryOfWideString itself."

	| charset |
	charset := (self charsetOf: inputs) ifNil: [self currentCharset].
	inputs keysAndValuesDo: 
			[:key :value | 
			inputs at: key
				put: (value collect: 
							[:each | 
							each convertToWithConverter: (TextConverter newForEncoding: charset)])].
	^inputs! !

!WebRetriever methodsFor: 'changing page' stamp: 'ich. 1/5/2006 19:20'!
retrieveFormWithInputs: inputs url: newUrl method: method encoding: encoding 
	| newSource |
	newSource := method asLowercase = 'get' 
				ifTrue: [newUrl retrieveContentsArgs: inputs]
				ifFalse: 
					[encoding = MIMEDocument contentTypeMultipart 
						ifTrue: [newUrl postMultipartFormArgs: inputs]
						ifFalse: [newUrl postFormArgs: inputs]].
	documentQueue nextPut: newSource! !

!WebRetriever methodsFor: 'private-document handling' stamp: 'ich. 1/3/2006 18:24'!
save: mimeDocument 
	| file |
	file := self openFileForSave.
	file ifNil: [^self].
	file reset;
		binary.
	file nextPutAll: mimeDocument content.
	file close! !

!WebRetriever methodsFor: 'private-html handling' stamp: 'ich. 1/8/2006 11:17'!
setBackGround
	| bgimageUrl bgimageDoc bgimage |
	(bgimageUrl := htmlDocument body background) ifNotNil: 
			[bgimageDoc := (bgimageUrl asUrlRelativeTo: currentUrl) retrieveContents.
			[bgimage := ImageReadWriter formFromStream: bgimageDoc contentStream binary] 
				ifError: 
					[:err :rcvr | 
					"ignore"

					bgimage := nil]].
	bgimage ifNotNil: [backgroundColor := bgimage]
		ifNil: [backgroundColor := Color fromString: htmlDocument body bgcolor]! !

!WebRetriever methodsFor: 'private-html handling' stamp: 'ich. 12/24/2005 11:11'!
setCurrentAnchorLocationWith: aFormatter
	currentAnchorLocation := currentUrl fragment
		ifNil: [nil]
		ifNotNil:
			[aFormatter anchorLocations
				at: currentUrl fragment asLowercase
				ifAbsent: [nil]].

! !

!WebRetriever methodsFor: 'private-document handling' stamp: 'ich. 1/3/2006 18:24'!
setCurrentPage: mimeDocument 
	currentUrl := mimeDocument url.
	pageSource := mimeDocument content.
	self changedAll: #(
				#currentUrl
				#hasLint
				#lint
				#backgroundColor
				#formattedPage
				#formattedPageSelection
			).
	self status: 'sittin' translated! !

!WebRetriever methodsFor: 'lint' stamp: 'ls 7/29/1998 00:07'!
showLint
	(StringHolder new contents: self lint) openLabel: 'lint for ', self currentUrl printString.! !

!WebRetriever methodsFor: 'stepping' stamp: 'ich. 1/3/2006 12:31'!
start
	self morph ifNotNil: [self morph start]! !

!WebRetriever methodsFor: 'private-download' stamp: 'ich. 1/11/2006 00:56'!
startDownloadingMorphState: morphs 
	| count |
	count := self workerCount min: morphs size.
	count <= 0 ifTrue: [^self].
	1 to: count
		do: 
			[:start | 
			| worker |
			worker := ScamperWorker 
						task: (morphs atAll: (start to: morphs size by: count))
						from: self.
			worker start]! !

!WebRetriever methodsFor: 'private-download' stamp: 'ls 8/12/1998 00:56'!
startDownloadingStateIn: aDocument  url: aUrl
	"download the state for the given document in a background thread.  signal the foreground when the data has arrived"
	downloadingProcess _ [	
		aDocument allSubentitiesDo: [ :e |
			e downloadState: aUrl ].
		documentQueue nextPut: #stateDownloaded.
		downloadingProcess _ nil. ] newProcess.
	downloadingProcess resume.! !

!WebRetriever methodsFor: 'private-download' stamp: 'ich. 12/29/2005 20:55'!
startDownload: newUrl 
	self status: ('downloading {1}...' translated format: { (newUrl toText) }).
	downloadingProcess := 
			[self download: newUrl.
			downloadingProcess := nil] newProcess.
	downloadingProcess resume! !

!WebRetriever methodsFor: 'private-download' stamp: 'ich. 1/9/2006 00:04'!
startRetrieveFromWithInputs: inputs url: newUrl method: method encoding: encoding 
	self status: 'submitting form...' translated.
	self restoreEncodingOf: inputs.	"MultiString inputs is restored to proper charset."
	downloadingProcess := 
			[self 
				retrieveFormWithInputs: inputs
				url: newUrl
				method: method
				encoding: encoding.
			downloadingProcess := nil] 
					newProcess.
	downloadingProcess resume! !

!WebRetriever methodsFor: 'accessing' stamp: 'ls 7/16/1998 22:18'!
status
	^status! !

!WebRetriever methodsFor: 'accessing' stamp: 'ls 7/16/1998 22:18'!
status: aSymbol
	status _ aSymbol.
	self changed: #status.! !

!WebRetriever methodsFor: 'stepping' stamp: 'ich. 1/4/2006 20:58'!
step
	"check if a new document has arrived"

	| results |
	[documentQueue isEmpty] whileFalse: 
			[results := documentQueue next.
			results == #stateDownloaded 
				ifTrue: [self reformatPage]
				ifFalse: [self storeAndDisplay: results]]! !

!WebRetriever methodsFor: 'changing page' stamp: 'ich. 12/25/2005 01:20'!
stopEverything
	"bring this Scamper to a sane state before embarking on something new"
	
	downloadingProcess ifNotNil:
		[downloadingProcess terminate.
		downloadingProcess := nil. ].
	[documentQueue isEmpty] whileFalse: [documentQueue next].
	self status: 'sittin' translated.
! !

!WebRetriever methodsFor: 'document handling' stamp: 'ich. 1/4/2006 20:58'!
storeAndDisplay: mimeDocument 
	history add: mimeDocument.
	self display: mimeDocument! !

!WebRetriever methodsFor: 'changing page' stamp: 'ich. 12/19/2005 00:29'!
submitFormWithInputs: inputs  url: url  method: method
	^ self submitFormWithInputs: inputs  url: url  method: method encoding: nil.
! !

!WebRetriever methodsFor: 'changing page' stamp: 'ich. 1/8/2006 12:12'!
submitFormWithInputs: inputs url: url method: method encoding: encoding 
	self stopEverything.
	(self checkFormUrl: url andMethod: method) ifFalse: [^false].
	self 
		startRetrieveFromWithInputs: inputs
		url: (url asUrlRelativeTo: currentUrl)
		method: method
		encoding: encoding.
	self start.
	^true! !

!WebRetriever methodsFor: 'accessing' stamp: 'ich. 1/5/2006 16:35'!
textConverter
	^textConverter! !

!WebRetriever methodsFor: 'accessing' stamp: 'ich. 1/5/2006 16:35'!
textConverter: anObject
	textConverter := anObject! !

!WebRetriever methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:46'!
viewSource
	"view the source HTML of this page"
	(StringHolder new contents: (pageSource withSqueakLineEndings)) openLabel: ('source for {1}' translated format: {currentUrl printString}).! !

!WebRetriever methodsFor: 'menus' stamp: 'ich. 12/25/2005 01:20'!
visitStartPage
	self jumpToAbsoluteUrl: 'browser:start'.
	^true! !

!WebRetriever methodsFor: 'stepping' stamp: 'di 1/14/1999 09:00'!
wantsSteps
	^ true! !

!WebRetriever methodsFor: 'private' stamp: 'ich. 1/3/2006 16:18'!
webBrowser
	^ self dependents detect: [:each | each isKindOf: Model] ifNone: [nil].
! !

!WebRetriever methodsFor: 'private' stamp: 'ich. 1/11/2006 00:57'!
workerCount
	^workerCountEdit contents string asInteger! !

!WebRetriever methodsFor: 'private' stamp: 'ich. 1/11/2006 00:57'!
workerCountEdit
	workerCountEdit := TextMorph new crAction: 
					[workerCountEdit contents string isAllDigits 
						ifTrue: [workerCountEdit acceptContents]
						ifFalse: 
							[workerCountEdit contents: self defaultWorkerCount asString.
							self resetWorkerCount]].
	self resetWorkerCount.
	^workerCountEdit! !

!HtmlParser class methodsFor: '*scamperm' stamp: 'ich 1/21/2006 11:25'!
parseTokens: tokenStream
	|  entityStack document token matchesAnything entity isScript |
	entityStack _ OrderedCollection new.

	"set up initial stack"
	document _ HtmlDocument new.
	entityStack add: document.
	isScript := false.
	
	"go through the tokens, one by one"
	[(token _ tokenStream next) isNil] whileFalse:
		[(token isTag and: [token isNegated])
			ifTrue:
				[matchesAnything _ (entityStack detect: [:e | e tagName = token name] ifNone: [nil]) isNil not.
				matchesAnything ifTrue:
					["pop the stack until we find the right one"
					[entityStack last tagName ~= token name] whileTrue: [entityStack removeLast].
					(entityStack removeLast isKindOf: HtmlScript) ifTrue: [isScript _ false]]]
			ifFalse:
				[isScript
					ifTrue:
						[entity _ HtmlCommentEntity new initializeWithText: token source]
					ifFalse:
						[token isComment ifTrue:
							[entity _ HtmlCommentEntity new initializeWithText: token source].
						token isText ifTrue:
							[entity _ HtmlTextEntity new text: token text.
							(((entityStack last shouldContain: entity) not) and: [token source isAllSeparators]) ifTrue: [entity _ nil]].
						token isTag ifTrue:
							[entity _ token entityFor.
							(entity isKindOf: HtmlScript) ifTrue: [isScript _ true]].
						entity ifNotNil:
							[entity isComment
								ifTrue:
									["just stick it anywhere"
									entityStack last addEntity: entity ]
								ifFalse:
									["only put it in something that is valid"
									[entityStack last mayContain: entity]  whileFalse: [ entityStack removeLast ].
									entityStack last addEntity: entity.
									entityStack addLast: entity]]]]].

	document parsingFinished.

	^document! !

!HtmlTokenizer methodsFor: '*scamperm' stamp: 'ls 8/20/1998 05:34'!
nextText
	"returns the next textual segment"
	|start end|

	start _ pos.
	end _ (text indexOf: $< startingAt: start ifAbsent: [ text size + 1 ]) - 1.

	pos _ end+1.
	^HtmlText forSource: (text copyFrom: start to: end)! !
ScamperM initialize!
WebRetriever initialize!
